# --- fichier SMB.pm ---
package ESIA::SMB;
use strict;
use warnings;
use DateTime::Format::Strptime;
use File::Path;
use File::Temp qw/ tempfile tempdir /;
use Try::Tiny;

sub smbclient_init
{
    my $local_env = 'C.UTF-8';
    my ( $url, $authfile, $options ) = @_;
    my %smb_params;

    my ( $ret , $stdout, $stderr )= cmd_run("bash -c 'export LC_ALL=$local_env'");

    if ( $stderr ne "" )
    {
        die "Unknown required local $local_env : $stderr\n";
    }

    $smb_params{'path'} = "export LC_ALL=$local_env;/usr/bin/smbclient";
    $smb_params{'args'} = "$url -A $authfile";
    return %smb_params;
}

sub smbclient_ls
{
    my %smb_params  = %{$_[0]};
    shift;
    my ( $directory, $wildcard ) = @_;

    if( !defined $wildcard )
    {
        $wildcard = '';
    }
    else
    {
        $wildcard = '"'.$wildcard.'"';
    }

    my $path = $smb_params{'path'};
    my $args = $smb_params{'args'};

    if ($directory ne "")
    {
    	$args = "$args -D $directory";
    }

    my $cmd = "$path $args -c 'ls $wildcard'";
    my ( $ret , $stdout, $stderr )= cmd_run($cmd);
    if( $ret != 0 )
    {
		print "$cmd\n";
        die $stderr;
    }

    my @rows = split("\n", $stdout);
    my @ret;
    foreach(@rows)
    {
       my ($file_name, $file_type, $file_size,$file_date ) = ( $_ =~ '\s+(.*)\s(\w)\w?\s+(\d+)\s+(.*)');

       if ( defined $file_size )
       {
           $file_name =~ s/\s+$//;
           $file_date =~ s/\s+$//;
           my %file_fields;
           $file_fields{'name'} = $file_name;

           if ( $file_type eq 'D' )
           {
                $file_fields{'type'} = 'D'; #Directory
           }
           elsif ( $file_type eq 'N' )
           {
                $file_fields{'type'} = 'F'; #File
           }
           elsif ( $file_type eq 'A' )
           {
                $file_fields{'type'} = 'F'; #File
           }
           else
           {
                $file_fields{'type'} = 'U'; #Unset/Unknown
           }
		   $file_fields{'date'} = $file_date;
           #$file_fields{'size'} = $file_size; Pas fiable ici pour les directory. Pourrait retourner 0 ou la bonne valeur.
           push @ret, \%file_fields;
        }
    }
    return @ret;
}

sub smbclient_file_info
{
    my %smb_params  = %{$_[0]};
    shift;
    my ( $directory, $namefile ) = @_;

    my $c_namefile = '"'.$namefile.'"';

    my $path = $smb_params{'path'};
    my $args = $smb_params{'args'};

    my $cmd = "$path $args -D $directory -c 'allinfo $c_namefile'";


    my ( $ret , $stdout, $stderr )= cmd_run($cmd);

    if( $ret != 0 )
    {
        die $stderr;
    }

    my @rows = split("\n", $stdout);
    my %ret;
    foreach(@rows)
    {
        my ($key, $value ) = ( $_ =~ '^(.*):\s+(.*)$');
        if ( defined $value )
        {
            $ret{'name'} = $namefile;
            if( $key eq 'altname' )
            {
                next;
            }
            elsif( $key eq 'attributes' )
            {
               my ( $num_type ) = ( $value =~ '\w?\s?\((\d\d)\)' );
               print "$value\n";
               if ( $num_type == 10 )
               {
                    $ret{'type'} = 'D'; #Directory
               }
               elsif ( $num_type == 80 )
               {
                    $ret{'type'} = 'F'; #File
               }
               else
               {
                    $ret{'type'} = 'U'; #Unset/Unknown
               }
            }
            elsif ( $key eq 'stream' ) #Les répertoires ne possédent pas cette clé
            {
                my ( $bytes )= ( $value =~ '(\d+)\s+byte' );
                $ret{'size'} = $bytes;
            }
            else
            {
                $ret{$key} = $value;
                my @type_date = split(/_/, $key );
                my $ts_key = $type_date[0] . '_'.'ts';

                my $strp = DateTime::Format::Strptime->new(
                    pattern   => '%a %h %e %H:%M:%S %p %Y %Z',
                );
                my $dt = $strp->parse_datetime($value);

                $ret{$ts_key} = $dt->epoch;
            }
        }
    }
    return %ret;
}

sub smbclient_clean
{
    my %smb_params  = %{$_[0]};
    if( defined $smb_params{'tmp_file'} )
    {
        #cmd_clean($smb_params{'tmp_file'});
    }
}

sub mountcifs_init
{
    my ( $url, $authfile, $options ) = @_;
    my $cmd;
    my $ret;
    my $stderr;
    my $stdout;

    $cmd = "grep '$url' /proc/mounts";
    ( $ret , $stdout, $stderr )= cmd_run($cmd);

    if ( $ret != 1 )
    {
        die "The share $ret $url is already mounted, $stdout";
    }

    my $mount_name = File::Temp->new(TEMPLATE => '/tmp/mountcifs_XXXXX').'A';

    mkpath($mount_name) or die "Couldn't create $mount_name directory, $!";

    $cmd = "mount.cifs $url $mount_name -o guest,credentials=$authfile";
    ( $ret , $stdout, $stderr )= cmd_run($cmd);

    if( $ret != 0 )
    {
        die $stderr;
    }

    my %smb_params;
    $smb_params{'mount_name'} = $mount_name;
    return %smb_params;
}

sub mountcifs_ls
{

}

sub mountcifs_tree
{

}

sub mountcifs_file_info
{

}

sub mountcifs_size
{

}

sub mountcifs_clean
{   return;
    my %smb_params  = %{$_[0]};
    my $mount_name  = $smb_params{'mount_name'};

    my $cmd = "umount $mount_name";
    my ( $ret , $stdout, $stderr )= cmd_run($cmd);

    if( $ret != 0 )
    {
        die $stderr;
    }
}

sub cmd_run()
{
    my ( $cmd ) = @_;
    my $tmp_dir = tempdir( CLEANUP => 1 ); # Le répertoire sera effacé automatiquement une fois le script exécuté

    my $stdout_file = "$tmp_dir/stdout";
    my $stderr_file = "$tmp_dir/stderr";

    system("$cmd > $stdout_file 2> $stderr_file");

    my $stdout = '';
    open(FH, '<', "$stdout_file") or die "Can't open < $stdout_file: $!";;
    while(<FH>){
        $stdout .= $_;
    }
    close(FH);

    my $stderr = '';
    open(FH, '<', "$stderr_file") or die "Can't open < $stderr_file: $!";;
    while(<FH>){
        $stderr .= $_;
    }
    close(FH);

    my @ret;
    if ($? == -1) {
        die "failed to execute: $!\n";
    }
    elsif ($? & 127) {
        die "child died with signal %d, %s coredump\n", ($? & 127),  ($? & 128) ? 'with' : 'without';
    }
    else {
        $ret[0] = ( $? >> 8 );
    }
    $ret[1] = $stdout;
    $ret[2] = $stderr;
    return @ret;
}

1;
