<?php
namespace ESIA\IP;
function execIPCommand($debug,$params)
{
	$execpath= PATH_IP_COMMAND.' '.$params;
	Trace($debug,"EXEC $execpath");
	if(exec($execpath, $output,$ret) === false)
	{
		throw new Exception("Exec Error: ".$execpath);
	}
}
function execDHClient($debug,$name)
{
	$execpath= PATH_DHCLIENT.' '.$params;
	Trace($debug,"EXEC $execpath");
	if(exec($execpath, $output,$ret) === false)
	{
		throw new Exception("Exec Error: ".$execpath);
	}
}
function enableSpecificRoute($debug,$inter,$ips)
{
	$ips = explode(',',$ips);
	$local_net_info = "";
	if (filter_var($inter['ip_addr'],  FILTER_VALIDATE_IP,FILTER_FLAG_IPV4))
	{
		$local_net_info = ipv4NetworkInformation($inter['ip_addr'], $inter['netmask']);
	}
	else
	{
		$local_net_info = ipv6NetworkInformation($inter['ip_addr'], $inter['netmask']);
	}
	foreach($ips as $ip)
	{
		$notSameNetwork = FALSE;
		$ip_version = 0;
		if (filter_var($ip,  FILTER_VALIDATE_IP,FILTER_FLAG_IPV4))
		{
			$ip_version = 4;
			Trace($debug,"VALID IPV4  $ip");
			$decip = ip2long($ip);
			$local_net = $local_net_info['network'];
			$local_br = $local_net_info['broadcast'];
			if($decip > $local_net && $decip < $local_br)
			{
				//on est dnas le même réseau local. il ne faut pas toucher au routage
				Trace($debug,"IP $ip is in same local network IPV4");
			}
			else
			{
				$notSameNetwork=TRUE;
			}
		}
		elseif (filter_var($ip,  FILTER_VALIDATE_IP,FILTER_FLAG_IPV6))
		{
			$ip_version = 6;
			Trace($debug,"VALID IPV6  $ip");
			$decip = ipv6ToBin($ip);
			$local_net = $local_net_info['network'];
			$local_br = $local_net_info['broadcast'];
			if($decip > $local_net && $decip < $local_br)
			{
				//on est dnas le même réseau local. il ne faut pas toucher au routage
				Trace($debug,"IP $ip is in same local network IPV6");
			}
			else
			{
				$notSameNetwork=TRUE;
			}
		}
		if($notSameNetwork==TRUE)
		{
			Trace($debug,"Modification du routage $ip");
			//modification ARP ou passerelle
			$remote_addr = $ip;
			$local_iface = $inter['name'];

			if($inter['vlan']!=0 && !empty($inter['vlan']))
			{
				$local_iface .= '.' . $inter['vlan'];
			}

			$local_gw = $inter['gateway'];
			if( $ip_version == 4 )
			{
				$cmd = "ip route add $remote_addr/32 dev $local_iface via $local_gw";
				echo "$cmd\n";
				if(exec($cmd, $output,$ret) === false)
				{
					throw new Exception("Exec Error: ".$cmd);
				}
			}
			elseif( $ip_version == 6 )
			{
				$cmd = "ip -6 route add $remote_addr/128 dev $local_iface via $local_gw";
				if(exec($cmd, $output,$ret) === false)
				{
					throw new Exception("Exec Error: ".$cmd);
				}
			}
			else
			{
				Trace($debug,"Unknown ip version $ip_version");
			}
		}

	}
}

function ipv4NetworkInformation ($ip_address, $netmask)
{
    if( is_numeric($netmask) ) //Cidr
    {
        $ip_nmask_long = 32 - $netmask;
        $ip_broadcast_invert = pow(2,$ip_nmask_long)-1;
        $ip_nmask_long = ip2long('255.255.255.255')-$ip_broadcast_invert;
    }
    else //netmask string
    {
        $ip_nmask_long = ip2long($netmask);
        $ip_broadcast_invert = ip2long('255.255.255.255')-$ip_nmask_long;
    }
    //convertion des IPv4 en décimal
    $ip_address_long = ip2long($ip_address);
    //On fait le & pour avoir l'ip du réseau en décimal
    $ip_net = $ip_address_long & $ip_nmask_long;
    //calcul de l'adresse de broadcast
    $ip_broadcast = $ip_address_long | $ip_broadcast_invert;
    return array('network'=>$ip_net,'broadcast'=> $ip_broadcast);
}
function ipv6NetworkInformation ($ip_address, $cidr)
{
    //convertion des IPv6 en binaire
    $binari6 = ipv6ToBin($ip_address);
	$hosts = (128 -  $cidr);
    $networks = 128 - $hosts;
	$_m = str_repeat('1', $networks).str_repeat('0', $hosts);
	$_hexMask = null;
	foreach(str_split($_m, 4) as $segment)
	{
    	$_hexMask .= base_convert($segment, 2, 16);
	}
	$netmask = substr(preg_replace('/([A-f0-9]{4})/', '$1:', $_hexMask), 0, -1);
	$binari_netmask = ipv6ToBin($netmask);


	$network = $binari6 & $binari_netmask;
    //calcul de l'adresse de broadcast
    $ip_broadcast_invert = ~$binari_netmask;
    $ip_broadcast = $network | $ip_broadcast_invert;

	return array('network'=>$network,'broadcast'=> $ip_broadcast);
}
function ipv6ToBin($ipaddress)
{
    $pton = @inet_pton($ipaddress);
    if (!$pton)
    {
    	return false;
    }
    $binari6 = current(unpack('a*',$pton));
	return  $binari6;
}
function mask2cidr($mask)
{
	if(is_numeric($mask))
	{
		return $mask;
	}
	$long = ip2long($mask);
	$base = ip2long('255.255.255.255');
  	return 32-log(($long ^ $base)+1,2);
}
