<?php
include_once '/usr/local/esia/plugins/lib/php/HTTP.class.php';

/**
    @file
    @brief Permet simplifier l'envoi de requête vers une API Proxmox
    @details
		La classe ProxmoxClient de ce fichier étends la classe HTTPClient en redéfinissant
		certaines méthodes pour simplifier une utilisation avec l'API Proxmox.
    @author Julien George
*/

class ProxmoxClient extends ESIA\HTTP\HttpClient
{
	protected $driverSettings;
	protected $request_counter=0;
	protected $connection_recycled = TRUE;
	protected $autoopen;
	protected $ticketfile=NULL;

	public function __construct($settings)
	{
		$this->driverSettings = $settings;
		$url = $settings['url'];

		$path_prefix = 'api2/json';
		$url = "$url/$path_prefix";

		parent::__construct($url);
	}

	public function open($use_ticket = TRUE)
	{
		$settings = $this->driverSettings;
		$username = $settings['username'];
		$token_name = $settings['token_name'];
		$token_value = $settings['token_value'];

		$token_id = "$username!$token_name";

		$header = array();
		$header[] = "Authorization: PBSAPIToken=$token_id:$token_value";

		parent::open();
		$this->setoptions(CURLOPT_HTTPHEADER,$header);
	}


	public function execute_request($options = null)
	{
		$data = parent::execute_request($options);
		if ( !in_array($this->getHttpCode(),array(200,201,204)))
		{
			if( $this->getHttpCode() == 401 && $this->ticketfile !=NULL && $this->connection_recycled ){
				unlink($this->ticketfile);
				if( $this->autoopen ){
					$this->close();
					$this->open(FALSE);
					return $this->execute_request($options);
				}
				else{
					throw new Exception('Http error : 401 Usage of expired proxmox ticket');
				}
			}
			throw new Exception('Http error : '.$this->getHttpCode(). " $data");
		}

		$this->request_counter++;

		if (empty ($data))
			return NULL;
		$data = json_decode($data, TRUE);
		if ($data === FALSE)
			throw new Exception('Unable to parse json response');
		return $data['data'];
	}
}
