#!/usr/bin/php
<?php
define('PLUGIN_BIN',basename(__FILE__));

include_once "VeeamClient.php";
include_once('/usr/local/esia/plugins/lib/php/backup/drivers/common/common.lib.php');

setSettings('url,username,password,proxyserver',$action,$options);

$options['api_version'] = 'v7';

//Si on à l'options gesasn, c'est qu'on doit passer à travers une unity
if( isset($options['gesasn'])){
	$output = array();
	$ret = NULL;
	$success = unityRun(PLUGIN_BIN,$action,$options,$output,$ret);
	echo $output;
	exit($ret);
}

$short_id = md5($options['url'].$options['username'].$options['proxyserver']);
$tmp_file_dir = "/tmp/esia/backup/veeam365/$short_id";

$result = array();
switch($action){
	case 'getjobsids' :
		$ret = getJobsIDS($result,$options);
		break;
	case 'getjobsvalues' :
		$ret = getJobsInfos($result,$options);
		break;
	default :
		echo "Method $action not implemented\n"; exit($EXIT_NOT_IMPLEMENTED);
}

echo json_encode($result,JSON_PRETTY_PRINT)."\n";
exit($ret);

function getJobsIds(&$result,$options)
{
	if( isset($options['status']) && strtolower($options['status']) != 'all')
	{
		$opt_status = $options['status'];
	}
	else
	{
		$opt_status = '0,1,2,3';
	}
	$opt_status = explode(',',$opt_status);

    $params['limit'] = 10000;
    $period_ts = period_to_ts($options["period"]);
    $date = date('Y-m-d\TH:i:s\Z',$period_ts);

	$ret = array();
    try
    {
        $client = new VeeamClient($options);
        $client->open();

		cleanDir();
		$proxyserver = load('Proxies',$client,$params);
		$currentproxy = FALSE;
		foreach($proxyserver as $row)
		{
			if( $row['hostName'] == $options['proxyserver'])
			{
				$currentproxy = $row;
			}
		}

		if( $currentproxy === FALSE)
		{
			echo "Proxy ".$options['proxyserver']." not found\n";
			foreach($proxyserver as $row)
			{
				echo $row['hostName']."\n";
			}
	        return $GLOBALS['EXIT_ERROR'];
		}

		$params['endTimeLowerBound'] = $date;
	//	$backups = $client->get('/JobSessions',$params);

		$backups = load('JobSessions',$client,$params);

		$current_jobs = array();
		if( !empty($backups))
		{
			load('organizations',$client,$params); //Pour + de perfs dans getValues()
			$jobs = load('Jobs',$client,$params);

			$proxy_id = $currentproxy['id'];
			$params['proxyId'] = $proxy_id;
			$repositories = load('backuprepositories',$client,$params);

			$current_repo = array();
			foreach($repositories as $repo)
			{
				$current_repo[] = $repo['_links']['self']['href'];
			}
			$current_job = array();
			foreach($jobs as $job)
			{
				if(in_array($job['_links']['backupRepository']['href'],$current_repo))
				{
					$current_jobs[] = $job['_links']['self']['href'];
				}

			}
		}
		foreach($backups as $backup)
		{
			$esia_status = getEsiaStatus($backup['status']);
			if( in_array($esia_status,$opt_status) && in_array($backup['_links']['job']['href'],$current_jobs) )
			{
				$ret[] = $backup['id'];
			}
		}
        $client->close();
    }
    catch(Exception $ex)
    {
        echo $ex->getMessage()."\n";
        return $GLOBALS['EXIT_ERROR'];
    }
	$result['ids'] = $ret;
	return $GLOBALS['EXIT_SUCCESS'];
}

function getEsiaStatus($backup_status)
{
	$status_esia['NotConfigured'] = 4;
	$status_esia['Queued'] = 4;
	$status_esia['Running'] = 4;

    $status_esia['Warning'] = 1;
    $status_esia['Error'] = 2;
    $status_esia['Success'] = 0;

	$status_esia['Stop'] = 2;
	$status_esia['Stopped'] = 2;

	$status_esia['Failed'] = 2;
	$status_esia['Disconnected'] = 2;


	return $status_esia[$backup_status];
}

function load($resource,$client,$params)
{
	$data = $client->get("/$resource",$params);
	if( isset($data['results']))
	{
		$data = $data['results'];
	}
	foreach($data as $row)
	{
		writeJobSessions($row['id'],$row);
	}
	return $data;
}

function take($resource,$id,$client,$params,$erase=FALSE)
{
	$data = readjobSession($id);
	if( empty($data) || !is_array($data))
	{
		if( !$client->isOpen())
		{
			$client->open();
		}
		$data = $client->get("/$resource/$id");
		if( isset($data['results']))
		{
			$data = $data['results'];
		}
	}
	elseif($erase)
	{
		eraseJobSession($id);
	}
	return $data;
}

function cleanDir()
{
	global $tmp_file_dir;
	$files = glob("$tmp_file_dir/*.json");
	foreach($files as $file)
	{
		unlink($file);
	}
}

function writeJobSessions($id,$data)
{
	global $tmp_file_dir;
	if( !is_dir($tmp_file_dir) )
	{
		mkdir($tmp_file_dir,0777,TRUE);
	}
	$data = json_encode($data);
	file_put_contents("$tmp_file_dir/$id.json",$data);
}

function readjobSession($id)
{
	global $tmp_file_dir;
	$raw = file_get_contents("$tmp_file_dir/$id.json");
	return json_decode($raw,TRUE);
}

function eraseJobSession($id)
{
	global $tmp_file_dir;
	unlink("$tmp_file_dir/$id.json");
}


function getJobsInfos(&$result,$options)
{
    $params['limit'] = 10000;

    try
    {
        $client = new VeeamClient($options); //On ouvre le client si nécessaire dans la function take().

	    $ids_list = explode(',',$options['ids']);

		$repositories = array();
		$organisations = array();
		$jobs = array();

	    $result = array();
	    foreach($ids_list as $id)
        {
			$backup = take('JobSessions',$id,$client,$params,TRUE);

            //Compatibilité v7
            if( !isset($backup['jobId']))
            {
                $backup['jobId'] = explode('/',$backup['_links']['job']['href']);
                $backup['jobId'] = $backup['jobId'][3];
            }

            if( empty($jobs[$backup['jobId']]))
            {
     			$jobs[$backup['jobId']] = take('Jobs',$backup['jobId'],$client,$params);
            }
            $job = $jobs[$backup['jobId']];

            //Compatibilité v7
            if( !isset($job['organizationId']))
            {
                $job['organizationId'] = explode('/',$job['_links']['organization']['href']);
                $job['organizationId'] = $job['organizationId'][3];
            }

            if( empty($organisations[$job['organizationId']]))
            {
                $organisations[$job['organizationId']] = take('organizations',$job['organizationId'],$client,$params);
            }
            $organisation = $organisations[$job['organizationId']];
            if( empty($repositories[$job['repositoryId']]))
            {
				$repositories[$job['repositoryId']] = take('backuprepositories',$job['repositoryId'],$client,$params);
            }
            $repositories = $repositories[$job['repositoryId']];
			$parse['intern_id'] = $id;
            $parse['name'] = $job['name'];
            $parse['source'] = $organisation['name'];
            $parse['storage'] =  $repositories['name'];
            $parse['size'] = $backup['statistics']['transferredDataBytes'];
            $parse['type'] = $backup['jobType'];
            $parse['level'] = $job['backupType'];
            $parse['status_backup'] = $backup['status'] . ' - ' . $backup['details'];
	        $parse['status_esia'] = getEsiaStatus($backup['status']);

	        $parse['start_time'] = date_format(new DateTime($backup['creationTime']),'U');
	        $parse['end_time'] = date_format(new DateTime($backup['endTime']),'U');
		    $result[] = $parse;
        }
    }
    catch(Exception $ex)
    {
        echo $ex->getMessage()."\n";
        return $GLOBALS['EXIT_ERROR'];
    }
	if( $client->isOpen())
	{
		$client->close();
	}
	return $GLOBALS['EXIT_SUCCESS'];
}
