<?php

namespace ESIA\plugins;

const ERRORS= array('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3);
const PRIORITIES= array('CRITICAL'=>2,'WARNING'=>1,'UNKNOWN'=>3,'OK'=>0);
const TIMEOUT = 50;
declare(ticks = 1);
class EsiaBadParameterType extends \Exception{}

/**
* Récupère les paramètres passés au script (-H, --hostname,etc)
* @param array $array un tableau de référence PHP
* @throws EsiaBadParameterType si un paramètre n'est pas du bon type
* @retval array un array associatifs retourné par getopt
*/

function getOptions(&$array)
{
	//Chargement des options courte
	$short = '';
	$long = array();
	$key_arr=array();
	foreach($array as $key => &$elem)
	{
		if(preg_match('#^([a-zA-Z])\=([a-z])$#',$key,$regs))
		{
			$short .=$regs[1].':';
			$key_arr[$regs[1]]=array('value'=>&$elem,'type'=>$regs[2]);
		}
		elseif(preg_match('#^([a-zA-Z])$#',$key,$regs))
		{
			$short.=$regs[1];
			$key_arr[$regs[1]]=array('value'=>&$elem,'type'=>'b');
		}
		elseif(preg_match('#^([a-zA-Z]*)\=([a-z])$#',$key,$regs))
		{
			$long[]=$regs[1].':';
			$key_arr[$regs[1]]=array('value'=>&$elem,'type'=>$regs[2]);
		}
		elseif(preg_match('#^([a-zA-Z]*)$#',$key,$regs))
		{
			$long[]=$regs[1];

			$key_arr[$regs[1]]=array('value'=>&$elem,'type'=>'b');
		}
	}
	$options = getopt($short, $long);
	foreach($options as $key => $opt)
	{
		if(array_key_exists($key,$key_arr))
		{
			if($key_arr[$key]['type']=='i' && !is_numeric($opt))
			{
				throw new EsiaBadParameterType("Bad type: '$key' option is not integer");
			}
			elseif($key_arr[$key]['type']=='f' && !is_numeric($opt))
			{
				throw new EsiaBadParameterType("Bad type: '$key' option is not float");
			}
			if($key_arr[$key]['type']=='b')
			{
				$key_arr[$key]['value']=TRUE;
			}
			else
			{
				$key_arr[$key]['value']=$opt;
			}
		}
	}
	return $options;

}
/**
* Calcule la priorité de sortie en une valeur de référence et la nouvelle
* @param integer $ref valeur actuelle
* @param integer $new futur valeur
* @throws /
* @retval integer la valeur la plus prioritaire entre les deux
*/
function priority_state($a,$b)
{
		#pas besoin d'aller dans la boucles si ils sont identiques
        if( $a == $b )
        {
        	return $a;
        }
		$priorities = \ESIA\plugins\PRIORITIES;
        foreach ($priorities as $v)
		{
                if ( $v == $a || $v == $b )
                {
                	return $v;
                }
        }
}
/**
* fonction par défaut de fin de programme quand le scipt met trop de temps à s'exécuter
* @param integer $signo ID du signal (par défaut SIGALARM)
* @throws /
* @retval /
*/
function exit_timeout($signo)
{
    echo "ERROR: Plugin took too long to complete (alarm)\n";
    exit (\ESIA\plugins\ERRORS["UNKNOWN"]);
}
/**
* Arme le SIGALARM pour cloturer le script après X seconde
* @param integer $timeout nombre de seconde avant la cloture du script
* @throws /
* @retval /
*/
function arm_end_signal($timeout = \ESIA\plugins\TIMEOUT)
{
    pcntl_alarm( $timeout );
    pcntl_signal(SIGALRM,'\ESIA\plugins\exit_timeout' );
}
/**
* Affiche une entête formatée
* @param string $progname nom du script
* @param string $progdescription(optionnel) description du script
* @throws /
* @retval /
*/
function print_header($progname,$progdescription=NULL)
{
	echo "Copyright (c) 2024 Esia SA\n";
	echo "\n";
	echo "$progname\n";
	$line = "\n";
	$line = str_pad($line,strlen($progname) , "=",STR_PAD_LEFT);
    echo $line;

	echo "\n";
    if(!empty($progdescription))
    {
        echo "Description:\n";
	    echo "     $progdescription\n";
		$line = "\n";
		$line=str_pad($line,strlen("     $progdescription") , "=",STR_PAD_LEFT);
    	echo $line;
    }
}
/**
* Affiche l'aide complet du script
* @param string $progname nom du script
* @param string $progdescription(optionnel) description du script
* @param array $vars(optionnel) liste des variables sous forme de tableau (une option = une ligne) array('h','hostname','ma super description')
* @throws /
* @retval /
*/
function print_help($progname,$progdescription=NULL,$vars=array())
{
	print_header($progname,$progdescription);
	if(!empty($vars))
	{
		echo "Plugin parameters:\n";
		$biggest_size_opt = 0;
		foreach($vars as $word)
		{
			$size = mb_strlen($word[1]);
			if($size > $biggest_size_opt)
			{
				$biggest_size_opt = $size;
			}
		}
		$biggest_size_opt = $biggest_size_opt+3;
		echo "   ".str_pad("short",7," ",STR_PAD_LEFT);
		echo "    ".str_pad("long",$biggest_size_opt," ",STR_PAD_RIGHT);
		echo " Description\n";
		foreach($vars as $var)
		{
			$short='';
			if(!empty($var[0]))
			{
				$short='-'.$var[0];
			}
			$short=str_pad($short,7," ",STR_PAD_LEFT);
			$long='';
			if(!empty($var[1]))
			{
				$long='--'.$var[1];
			}
			$long=str_pad($long,$biggest_size_opt," ",STR_PAD_RIGHT);
			echo" $short    $long   $var[2]\n";
		}
	}
}
/**
* Affiche ou pas un message de debug
* @param boolean $debug si égale à TRUE on affiche le message
* @param string|array $msg les message ou le tableau à afficher
* @throws /
* @retval /
*/
function trace($debug,$msg)
{
	if($debug==TRUE)
	{
		if(is_array($msg))
		{
			print_r($msg);
		}
		else
		{
			echo 'DEBUG: '.$msg."\n";
		}
	}
}
?>
