<?php
include_once 'common.class.php';

class VirtGesaConnectionFailed extends Exception
{
    public function __construct($message, $code = 0, Exception $previous = null)
    {
            parent::__construct($message, $code, $previous);
    }

    public function __toString()
    {
            return __CLASS__ . ": [{$this->code}]: {$this->message}\n";
    }
}

class RemoteDriver
{
    private $settings = NULL;

    public function __construct($settings)
    {
        $this->settings = $settings;
    }

    public function getSerialNumber()
    {
        return $this->settings['gesa_sn'];
    }

    /*@NAME: execByECATP
     *@IN: $command : la commande nrpe a exÃ©cuter
     *	$args (optionel) : des arguments a passer a la commande nrpe
     *	$timeout (optionel) : timeout en secondes
     *@OUT: un tableau associatif organiser comme suit:
     * 	$arr["status"]=END signigie que toutes les donnÃ©es on Ã©tÃ© transmises, MORE signifie qu'il reste des donnÃ©es.
     *	$arr["size"]=Nbre de lignes transmise, si ["status"]=MORE, correspond Ã  la ligne suivante Ã  transmettre.
     * 	$arr["data"]=Tableau retournant la totalitÃ© de l'output de checkn_nrpe, [0] correspondant a la premiÃ¨re ligne
     *@TREAT: ExÃ©cute un check ecatp et retourne un tableau avec les rÃ©sultats.
     *@EXCEPTION: VirtGesaConnectionFailed en cas d'erreur.
     *@TODO : Cette fonction est présente aussi dans la class gesa.class.php
     */
    public function execByECATP($command,$args="",$timeout=50,$accept_unknown=FALSE)
    {
        $sn = $this->getSerialNumber();
        exec('/usr/local/esia/plugins/check_gesa -G '. $sn . ' -C '. $command. ' -t '.$timeout." -A '$args'",$output,$returnexec);
      // 	echo '/usr/local/esia/plugins/check_gesa -G '. $sn . ' -C '. $command. ' -t '.$timeout." -A '$args'\n";
        if(empty($output[0]))
        {
            throw new VirtGesaConnectionFailed("GESA UNREACHEABLE ",0);
        }
        if(!$accept_unknown && preg_match('#UNKNOWN\s+ERROR#',$output[0]) || $returnexec==3 )
        {
            throw new VirtGesaConnectionFailed($output[0],1);
        }
        if(preg_match("#NRPE: Command $command not defined#",$output[0]))
        {
            throw new VirtGesaConnectionFailed($output[0],2);
        }
        //echo "<pre>".print_r($output,true)."</pre>";
        //check if json
        $firstchar = substr($output[0],0,1);
        if($firstchar=='{')
        {
            //on est en mode JSON
            $json="";
            foreach($output as $line)
            {
                $json.=$line;
            }
        //	echo($json);
            $json = @json_decode($json, true);
            if(is_array($json))
            {
                $data=array();
                foreach($json as $key => $value)
                {
                    $data[$key]=$value;
                }
                return array("status"=>'END',"size"=>count($json),"data"=>$data);
            }
            else
            {
                throw new VirtGesaConnectionFailed("BAD output return ",2);
            }
        }
        else
        {
            preg_match("#<(\w*)>#",end($output),$matches);
            if(empty($matches[1]))
            {
                return array("status"=>"CHECK",count($output),"data"=>$output);
            }

            array_pop($output);

            $data=array();
            foreach($output as $key => $value)
            {
                $data[$key]=str_getcsv ( $value , ",", '"','"');
            }
            return array("status"=>$matches[1],"size"=>count($output),"data"=>$data);
        }
    }

    public function execWithRetries($command,$args="",$timeout=50,$retries = 3)
    {
        $except = NULL;
        for($i=0;$i<$retries;$i++)
        {
            try
            {
                $ret = $this->execByECATP($command,$args,$timeout);
                return implode("\n",$ret['data']);
            }
            catch (\Exception $e)
            {
                $except = $e;
            }
            sleep(1);
        }
    }

    public function getResource($resource,$type,$id=FALSE)
    {
        if( $type == 'min' || $type == 'basic')
        {
            $type = "all";
        }

        $driver_name = $this->settings['driver'];
        $auth_file_name = $this->settings['auth_file'];
        $url = $this->settings['url'];

        $args = "-d $driver_name -A $auth_file_name -r $resource -t $type -u $url";
        if( $id )
        {
            $args .= " -m $id";
        }

        $settings = $this->settings;
        if( !isset($settings['retries']))
        {
            $settings['retries'] = 3;
        }
        if( !isset($settings['timeout']))
        {
            $settings['timeout'] = 20;
        }
        $data = $this->execWithRetries('get_virtualization.php',$args,$settings['timeout'],$settings['retries']);
        return json_decode($data,TRUE);
    }
}

class RemoteCluster extends Cluster
{
    private $driver = NULL;
    private $resources = array();
    private $hosts_list = array();
	private $storages_list = array();
    private $vms_list = array();
    private $vms_names = array();

    public function __construct($settings)
    {
        $this->driver = new RemoteDriver($settings);
    }

    private function getClusterResource($type,$key)
    {
        if( !isset($this->resources[$key]) )
        {
            $resources = $this->driver->getResource('cluster',$type);
            if( !is_array($resources))
            {
                throw new Exception('Unable to access to hypervisor');
            }

            $resources = array_shift($resources);
            foreach($resources as $idx => $values)
            {
                $this->resources[$idx] = $values;
            }
        }
        return $this->resources[$key];
    }

    public function getName()
    {
        return $this->getClusterResource('min','name');
    }

    public function getMasterHost()
    {
        return $this->getClusterResource('basic','master_host');
    }

    public function isCluster()
    {
        return $this->getClusterResource('basic','is_cluster');
    }

    public function isHA()
    {
        return $this->getClusterResource('basic','is_ha');
    }

    public function getCpusHA()
    {
        return $this->getClusterResource('metrics','cpus_ha');
    }

    public function getCpusUsage()
    {
        return $this->getClusterResource('metrics','cpus_usage');
    }

    public function getCpusTotal()
    {
        return $this->getClusterResource('metrics','cpus_total');
    }

    public function getMemHa()
    {
        return $this->getClusterResource('metrics','mem_ha');
    }

    public function getMemUsage()
    {
        return $this->getClusterResource('metrics','mem_usage');
    }

    public function getMemTotal()
    {
        return $this->getClusterResource('metrics','mem_total');
    }

    public function getNetInUsage()
    {
        return $this->getClusterResource('metrics','net_in');
    }

    public function getNetOutUsage()
    {
        $this->getHosts();
        return $this->getClusterResource('metrics','net_out');
    }

    public function getHosts():array
    {
        if( empty($this->hosts_list) )
        {
            $resources = $this->driver->getResource('host','min');
            foreach($resources as $host_raw)
            {
                $this->hosts_list[$host_raw['name']] = new RemoteHost($this->driver,$host_raw);
            }
        }
        return array_values($this->hosts_list);
    }

    public function getVMs():array
    {
        if( empty($this->vms_list) )
        {
            $resources = $this->driver->getResource('vm','min');
            foreach($resources as $vm_raw)
            {
                $this->vms_list[$vm_raw['id']] = new RemoteVM($this->driver,$vm_raw);
                if (!isset($this->vms_names[$vm_raw['name']]))
                {
                    $this->vms_names[$vm_raw['name']] = array();
                }
                $this->vms_names[$vm_raw['name']][] = $vm_raw['id'];
            }
        }
        return array_values($this->vms_list);
    }

    public function getTemplates():array
    {
        return array();
    }

    public function getStorages($shared = null):array
    {
		if( empty($this->storages_list) )
        {
            $resources = $this->driver->getResource('storage','min');
            foreach($resources as $sr_raw)
            {
                $this->storages_list[$sr_raw['id']] = new RemoteStorage($this->driver,$sr_raw);
            }
        }
		if( $shared == NULL )
		{
        	return array_values($this->storages_list);
		}
		else
		{
			$ret = array();
			foreach($this->storages_list as $row)
			{
				if( $row->isShared() == $shared)
				{
					$ret[] = $row;
				}
			}
			return $ret;
		}
    }

    public function getHostByName($hostname)
    {
        $this->getHosts(); //Pour charger en mémoire, si absente, la liste des hosts

        if( isset($this->hosts_list[$hostname]))
        {
            return $this->hosts_list[$hostname];
        }
        return NULL;
    }


    public function getVMFromSnapshot($snapid)
    {
        return NULL;
    }

    public function getVMByName($name)
    {
        $this->getVMs(); //Pour charger en mémoire, si absente, la liste des VMs
        $ret = array();
        foreach($this->vms_names[$name] as $id)
        {
            $ret[] = $this->vms_list[$id];
        }
        return $ret;
    }

    public function getVM($vmid,$refresh=FALSE)
    {
        $this->getVMs(); //Pour charger en mémoire, si absente, la liste des VMs
        if( isset($this->vms_list[$vmid]))
        {
            return $this->vms_list[$vmid];
        }
        return NULL;
    }
}

class RemoteHost extends Host
{
    private $resources = array();
    private $driver = NULL;
	private $vms_list = array();

    public function __construct($driver,$data_raw)
    {
        $this->driver = $driver;
        $this->setId($data_raw['id']);
        $this->setName($data_raw['name']);
        $this->resources = $data_raw;
    }

    private function getHostResource($type,$key)
    {
        if( !isset($this->resources[$key]) )
        {
            $resources = $this->driver->getResource('host',$type,$this->getId());

            $resources = array_shift($resources);
            foreach($resources as $idx => $values)
            {
                $this->resources[$idx] = $values;
            }
        }
        return $this->resources[$key];
    }

    public function getDescription()
    {
        return $this->getHostResource('basic','description');
    }

    public function getType()
    {
        return $this->getHostResource('basic','type');
    }

    public function getManufacturer()
    {
        return $this->getHostResource('basic','manufacturer');
    }

    public function getOS()
    {
        return $this->getHostResource('basic','os');
    }

    public function getOSVersion()
    {
        return $this->getHostResource('basic','os_version');
    }

    public function getAddress()
    {
        return $this->getHostResource('basic','address');
    }

    public function getUpTime()
    {
        return $this->getHostResource('basic','uptime');
    }

    public function isAlive()
    {
        return $this->getHostResource('basic','alive');
    }

    public function getCpusUsage()
    {
        return $this->getHostResource('metrics','cpus_usage');
    }

    public function getCpusTotal()
    {
        return $this->getHostResource('metrics','cpus_total');
    }

    public function getMemUsage()
    {
        return $this->getHostResource('metrics','mem_usage');
    }

    public function getMemTotal()
    {
        return $this->getHostResource('metrics','mem_total');
    }

    public function getNetInUsage()
    {
        return $this->getHostResource('metrics','net_in');
    }

    public function getNetOutUsage()
    {
        return $this->getHostResource('metrics','net_out');
    }

    //Host only
    public function getCpusHA()
    {
        return $this->getHostResource('metrics','cpus_ha');
    }

    public function getMemHA()
    {
        return $this->getHostResource('metrics','mem_ha');
    }

    public function getVMs()
    {
        if( empty($this->vms_list) )
        {
            $resources = $this->driver->getResource('host','vms');
            foreach($resources as $vm_raw)
            {
                $this->vms_list[$vm_raw['id']] = new RemoteVM($this->driver,$vm_raw);
            }
        }
        return array_values($this->vms_list);
    }
}

class RemoteVM extends VM
{
    private $resources = array();
    private $driver = NULL;
	private $volumes_list;
	private $snapshots_list;

    public function __construct($driver,$data_raw)
    {
        $this->driver = $driver;
        $this->setId($data_raw['id']);
        $this->setName($data_raw['name']);
        $this->resources = $data_raw;
    }

    private function getVMResource($type,$key)
    {
        if( !isset($this->resources[$key]) )
        {
            $resources = $this->driver->getResource('vm',$type,$this->getId());
            $resources = array_shift($resources);
            foreach($resources as $idx => $values)
            {
                $this->resources[$idx] = $values;
            }
        }
        return $this->resources[$key];
    }

    public function getDescription()
    {
        return $this->getVMResource('basic','description');
    }

    public function getType()
    {
        return $this->getVMResource('basic','type');
    }

    public function getManufacturer()
    {
        return $this->getVMResource('basic','manufacturer');
    }

    public function getOS()
    {
        return $this->getVMResource('basic','os');
    }

    public function getOSVersion()
    {
        return $this->getVMResource('basic','os_version');
    }

    public function getAddress()
    {
        return $this->getVMResource('basic','address');
    }

    public function getUpTime()
    {
        return $this->getVMResource('basic','uptime');
    }

    public function isAlive()
    {
        return $this->getVMResource('basic','alive');
    }

    public function getCpusUsage()
    {
        return $this->getVMResource('metrics','cpus_usage');
    }

    public function getCpusTotal()
    {
        return $this->getVMResource('metrics','cpus_total');
    }

    public function getMemUsage()
    {
        return $this->getVMResource('metrics','mem_usage');
    }

    public function getMemTotal()
    {
        return $this->getVMResource('metrics','mem_total');
    }

    public function getNetInUsage()
    {
        return $this->getVMResource('metrics','net_in');
    }

    public function getNetOutUsage()
    {
        return $this->getVMResource('metrics','net_out');
    }

    //VM only
    public function getState()
    {
        return $this->getVMResource('basic','state');
    }

    public function isLocked($refresh = false)
    {
        return $this->getVMResource('basic','is_locked');
    }

    public function isHA()
    {
        return $this->getVMResource('basic','is_ha');
    }

    public function getHASettings()
    {
        return $this->getVMResource('basic','ha_settings');
    }

    public function getLockStatus($refresh = false)
    {
        return $this->getVMResource('basic','lock_status');
    }

    public function getHost()
    {
        return $this->getVMResource('basic','host');
    }

    public function getCpuPlatform()
    {
        return $this->getVMResource('basic','cpu_platform');
    }

    public function isAgent()
    {
        return $this->getVMResource('basic','is_agent');
    }

    public function getVolumes()
    {
        if( empty($this->volumes_list) )
        {
            $this->volumes_list = $this->driver->getResource('vm','volumes',$this->getId());
			foreach($this->volumes_list as $key => $row)
			{
				$this->volumes_list[$key]['vm'] = $this;
				$this->volumes_list[$key]['storage'] = new RemoteStorage($this->driver,$row['storage']);
			}
        }
        return $this->volumes_list;
    }

    public function getSnapshots()
    {
        if( empty($this->snapshots_list) )
        {
            $this->snapshots_list = $this->driver->getResource('vm','snapshots',$this->getId());
        }
        return $this->snapshots_list;
    }

    public function getAllowedOperations($all = true)
    {
        return array();
    }

    public function getDiskRead()
    {
        return $this->getVMResource('metrics','disk_read');
    }

    public function getDiskWrite()
    {
        return $this->getVMResource('metrics','disk_write');
    }
}

Class RemoteStorage extends Storage
{
    private $resources = array();
    private $driver = NULL;
	private $volumes_list = array();

    public function __construct($driver,$data_raw)
    {
        $this->driver = $driver;
		$this->resources = $data_raw;
    }

    private function getStorageResource($type,$key)
    {
        if( !isset($this->resources[$key]) )
        {
            $resources = $this->driver->getResource('storage',$type,$this->resources['id']);
            $resources = array_shift($resources);
            foreach($resources as $idx => $values)
            {
                $this->resources[$idx] = $values;
            }
        }
        return $this->resources[$key];
    }

    public function getId()
    {
        return $this->getStorageResource('min','id');
    }

    public function getName()
    {
        return $this->getStorageResource('min','name');
    }

    public function isShared()
    {
        return $this->getStorageResource('basic','is_shared');
    }

    public function getContent()
    {
        return $this->getStorageResource('basic','content');
    }

    public function getType()
    {
        return $this->getStorageResource('basic','type');
    }

    public function getDiskRead()
    {
        return $this->getStorageResource('metrics','disk_read');
    }

    public function getDiskWrite()
    {
        return $this->getStorageResource('metrics','disk_write');
    }

    public function getSizeUsed()
    {
        return $this->getStorageResource('metrics','size_usage');
    }

    public function getSizeVirtual()
    {
        return $this->getStorageResource('metrics','size_virtual');
    }

    public function getSizeTotal()
    {
        return $this->getStorageResource('metrics','size_total');
    }

	public function getHosts()
	{
		$resources = $this->getStorageResource('metrics','hosts');
		$ret = array();
		foreach($resources as $row)
		{
			$ret[] = new RemoteHost($this->driver,$row);
		}
		return $ret;
	}

    public function getVolumes(){
        if( empty($this->volumes_list) )
        {
            $this->volumes_list = $this->driver->getResource('storage','volumes',$this->getId());
			foreach($this->volumes_list as $key => $row)
			{
				$this->volumes_list[$key]['vm'] = new RemoteVM($this->driver,$row['vm']);
				$this->volumes_list[$key]['storage'] = $this;
			}
        }
        return $this->volumes_list;
    }
}
