<?php
include_once "common/common.class.php";
include_once('hyperv/HypervClient.php');

class HyperVCluster extends Cluster
{
    protected $settings = NULL;
    protected $driver = NULL;

	protected static $time_divisor = 1;

	protected $hosts_list = NULL;
    protected $storages_list = NULL;
    protected $storages_path_list = NULL;
    protected $storages_vhd_list = NULL;

    protected $vms_list = NULL;
    protected $vms_id_from_name = NULL;

    public function __construct($settings)
	{
        HypervClient::$PERMANENT_TIME=3600;  //1h
        HypervClient::$LONG_TIME=360;   //5 min
        HypervClient::$SHORT_TIME=180; //1 min
        HypervClient::$MIN_DIFF_TIME=15; //1 min

        $this->settings = $settings;
        $this->driver = new HypervClient($settings);

        $o_host = $settings['url'];
        if( !$this->driver->testPortAccess() )
		{
            $list = $this->driver->getHostListAddress();
            $this->driver = NULL;
            if( is_array($list))
            {
                foreach($list as $addr)
                {
                    if( $o_host != $addr && HypervClient::testPowershellPortAccess($addr) )
                    {
                        $settings['url'] = $o_host;
                        $this->driver = new HypervClient($settings);
                    }
                }
            }
            if( $this->driver === NULL )
            {
			     throw new Exception("Unable to use powershell ports to access to hyperv hosts");
            }
		}
    }

    public function getDriver()
	{
        return $this->driver;
    }

    public function getName()
	{
	//	$resources = $this->getResourceData('cluster.info',HypervClient::$PERMANENT_TIME);
        $resources = $this->driver->getCluster();
        return $resources['data']['Name'];
    }
    public function getMasterHost()
	{
        return NULL;
    }

    public function isCluster()
	{
        $resources = $this->driver->getCluster();
		return ( !empty($resources['data']['Name']) );
    }

    public function isHA()
	{
        return $this->isCluster();
    }

    public function getHosts():array
	{
		if( $this->hosts_list == NULL )
		{
			$this->hosts_list = array();
            $resources = $this->driver->getHostList();
			foreach($resources['data'] as $host_raw)
			{
                $retRow['request_time'] = $resources['request_time'];
                $retRow['data'] = $host_raw;
                $this->hosts_list[$host_raw['Name']] = new HyperVHost($retRow,$this->settings,$this);
			}
		}
		return array_values($this->hosts_list);
    }

    public function getHostByName($name)
	{
        $this->getHosts(); //Pour charger les données.
        return $this->hosts_list[$name];
    }

    public function getVMs():array
	{
        if( $this->vms_list == NULL)
        {
            $host_list = $this->getHosts();
            foreach($host_list as $host)
            {
                $vm_list = $host->getVMs();
                foreach($vm_list as $vm)
                {
                    $vmid = $vm->getID();
                    $this->vms_list[$vmid] = $vm;
                    $this->vms_id_from_name[$vm->getName()][] = $vmid;
                }
            }
        }
        if( !is_array($this->vms_list) )
        {
            return array();
        }
        return array_values($this->vms_list);
    }

    public function getTemplates():array
	{
		return array();
    }

    public function getStorages($shared=NULL):array
    {
        $hosts = $this->getHosts();
        if( $this->storages_list == NULL )
        {
            $retList = array();
            foreach ($hosts as $host)
            {
                $resources = $host->getDriver()->getSRList($host->getName());
                foreach($resources['data'] as $row)
                {
                    if( !isset($retList[$row['id']]))
                    {
                        $retRow = array();
                        $retRow['request_time'] = $resources['request_time'];
                        $retRow['data'] = $row;
                        $retRow['hosts'] = array();
                        $retList[$row['id']] = $retRow;
                    }
                    $retList[$row['id']]['hosts'][] = $host;
                }
            }

            $this->storages_list = array();
            foreach($retList as $key => $row)
            {
                $this->storages_list[$key]['shared'] = $row['data']['isSharedVolume'];
                $this->storages_list[$key]['object'] = new HyperVStorage($row,$this->driver,$this);
            }
        }

        $ret = array();
        foreach( $this->storages_list as $key => $row )
        {
            if( $shared == NULL  || $row['shared'] == $shared )
            {
                $ret[] = $row['object'];
            }
        }
        return $ret;
    }

    public function getStoragesPathList()
    {
        $hosts = $this->getHosts();
        if( $this->storages_path_list == NULL )
        {
            foreach ($hosts as $host)
            {
                $resources =  $this->driver->getSRPath($host->getName());
                $this->storages_path_list[$host->getName()] = $resources;
            }
        }
        return $this->storages_path_list;
    }

    public function getVHDList()
    {
        $hosts = $this->getHosts();
        if( $this->storages_vhd_list == NULL )
        {
            foreach ($hosts as $host)
            {
                $resources = $this->driver->getSRVHD($host->getName());

                //On supprime les doublons dans les volumes (certains volumes sont récupérés sur tous les hosts)
                $computerName = $host->getComputerName();
                foreach($resources['data'] as $key => $vol)
                {
                    if( $vol['ComputerName'] != $computerName)
                    {
                        unset($resources['data'][$key]);
                    }
                }

                $this->storages_vhd_list[$host->getName()] = $resources;
            }
        }
        return $this->storages_vhd_list;
    }

    public function getVMFromSnapshot($snapid)
	{
        return NULL;
    }

    function getVMByName($name)
	{
        $this->getVMs();
        if( !isset($this->vms_id_from_name[$name]) )
        {
            return array();
        }
        $vm_list_id = $this->vms_id_from_name[$name];
        $ret = array();
        foreach($vm_list_id as $vm_id)
        {
            $ret[] = $this->getVM($vm_id);
        }
		return $ret;
    }

    public function getVM($vmid,$refresh=FALSE)
	{
        $this->getVMs();
        if( !isset($this->vms_list[$vmid]) )
        {
            return NULL;
        }
		return $this->vms_list[$vmid];
    }

    public function cloneVM($source,$target_name,$full=TRUE)
	{
		return NULL;
    }
}

class HyperVHost extends Host
{
    protected $cluster;
	protected $driver = NULL;
	protected $main_info;
	protected $host_values = NULL;
	protected $host_metrics = NULL;

    protected $vms_list = NULL;


    private $resources = array();
    private $isReacheable = NULL;

    public function __construct($main_info,$settings,$cluster)
	{
        $this->cluster = $cluster;
        $settings['url'] = $main_info['data']['mainAddress'];
        $this->driver = new HypervClient($settings);
		$this->main_info = $main_info;

        $this->setId($main_info['data']['UniqueID']);
        $this->setName($main_info['data']['Name']);
        //$this->setDescription('');
        $this->setType('hypervisor');
        $this->setManufacturer('Microsoft');
    }

    public function isReacheable()
    {
        if( $this->isReacheable === NULL )
        {
            $this->isReacheable = $this->driver->testPortAccess();
        }
        return $this->isReacheable;
    }

    public function getCluster()
    {
        return $this->cluster;
    }

    public function getDriver()
    {
        return $this->driver;
    }

    private function getHostValues()
    {
        if( !isset($this->resources['values']))
        {
            $name = $this->getName();
            $this->resources['values'] = $this->driver->getHostValues($name);
        }
        return $this->resources['values'];
    }

    public function getComputerName()
    {
        $resources = $this->getHostValues();
        return $resources['data']['ComputerName'];
    }


    private function getHostInterfaces()
    {
        if( !isset($this->resources['ifaces']))
        {
            $name = $this->getName();
            $data = $this->driver->getHostInterfaces($name);
            $data = $data['data'];

            //On trie les 2 données (adapter & address IP) par rapport a leur iface_name
            $netAdapter = array();
            foreach($data['netadapter']  as $adapter)
            {
                $netAdapter[$adapter['Name']] = $adapter;
            }
            $netAddress = array();
            foreach($data['netaddress'] as $addr)
            {
                if( !isset($netAddress[$addr['InterfaceAlias']]))
                {
                    $netAddress[$addr['InterfaceAlias']] = array();
                }
                $netAddress[$addr['InterfaceAlias']][] = $addr;
            }

            //On merge les données dans le même tableau
            //On récupérer aussi les adresses MACS (pour ajouter ensuite les ifaces sans IPs)
            $merged = array();
            $rest = array();
            $macList = array();
            foreach($netAdapter as $name => $adapter)
            {
                if( isset($netAddress[$name]) )
                {
                    $merged[$name] = $adapter;
                    $merged[$name]['addresses'] = $netAddress[$name];
                    $macList[$adapter['LinkLayerAddress']] = TRUE;
                }
                else if(!empty($adapter['LinkLayerAddress']))
                {
                    $rest[$adapter['LinkLayerAddress']] = $adapter;
                }
            }
            // On ajoute ce qui reste
            foreach($rest as $mac => $adapter)
            {
                if( !isset($macList[$mac]) )
                {
                    $merged[$adapter['Name']] = $adapter;
                    $merged[$name]['addresses'] = array();
                }
            }
            $this->resources['ifaces'] = $merged;
        }
        return $this->resources['ifaces'];
    }


    public function getVMValues($vm_id)
    {
        if( !isset($this->resources['vm_values']))
        {
            $name = $this->getName();
            $data = $this->driver->getVMValues($name);
            foreach($data['data'] as $row)
            {
                $row['request_time'] = $data['request_time'];
                $this->resources['vm_values'][$row['vmid']] = $row;

            }
        }
        if( isset($this->resources['vm_values'][$vm_id]))
        {
            return $this->resources['vm_values'][$vm_id];
        }
        else
        {
            return NULL;
        }
    }

    public function getVMSnapshots($vm_id)
    {
        if( !isset($this->resources['snapshots']))
        {
            $name = $this->getName();
            $data = $this->driver->getVMSnapshots($name);

            foreach($data['data'] as $row)
            {
                $this->resources['snapshots'][$row['VMId']][$row['Id']] = $row;
            }
        }

        if( isset($this->resources['snapshots'][$vm_id]))
        {
            return $this->resources['snapshots'][$vm_id];
        }
        else
        {
            return array();
        }
    }

    public function getVMMetrics($vm_id)
    {
        if( !isset($this->resources['vm_metric']))
        {
            $name = $this->getName();
            $data = $this->driver->getVMMetrics($name);
            foreach($data['data'] as $row)
            {
                if(isset($row['MeteringDuration']['TotalSeconds']) && $row['MeteringDuration']['TotalSeconds'] > 10)
                {
                    $this->resources['vm_metric'][$row['VMId']] = $row;
                }
            }
        }
        if( isset($this->resources['vm_metric'][$vm_id]))
        {
            return $this->resources['vm_metric'][$vm_id];
        }
        else
        {
            return NULL;
        }
    }

	public function getOS()
	{
        $resources = $this->getHostValues();
        $os = str_replace('Microsoft','',$resources['data']['os'] ?? '');
		return trim($os);
	}

	public function getOSVersion()
	{
        $resources = $this->getHostValues();
		return $resources['data']['os_version'] ?? '';
	}

    public function getDescription()
    {
        $resources = $this->getHostValues();
        return $resources['data']['description'] ?? '';
    }

    public function getAddress()
    {
        return $this->main_info['data']['mainAddress'] ?? '';
    }

    public function getUpTime()
    {
        $resources = $this->getHostValues();
        if( empty($resources['data']['uptime']) )
        {
            return 0;
        }
        $time = time();
        $request_time = floor(array_sum($resources['request_time'])/2);
        $last_exec = $time - $request_time;
        return $resources['data']['uptime'] + $last_exec;
    }

    public function isAlive()
    {
        if( !$this->isReacheable() )
        {
            return FALSE;
        }
        return (trim($this->main_info['data']['State']) == 'Up' && trim($this->main_info['data']['StatusInformation']) == 'Normal');
    }

    public function getCpusUsage()
    {
        if( !isset($this->resources['CPU']))
        {
            $name = $this->getName();
            $this->resources['CPU'] = $this->driver->getHostCPU($name);
        }
        return $this->resources['CPU'];
    }

    public function getCpusTotal()
    {
        $resources = $this->getHostValues();
		return $resources['data']['cpu_cores'] ?? 0;
    }

    public function getMemUsage()
    {
        $resources = $this->getHostValues();
        return ($resources['data']['mem_usage'] ?? 0)*1024;
    }

    public function getMemTotal()
    {
        $resources = $this->getHostValues();
        return ($resources['data']['mem_total'] ?? 0)*1024;
    }

    public function getNetInUsage()
    {
        $iface_list = $this->getNetInterfaces();
        return $iface_list['global']['rx'] ?? 0;
    }

    public function getNetOutUsage()
    {
        $iface_list = $this->getNetInterfaces();
        return $iface_list['global']['tx'] ?? 0;
    }

    public function getNetInterfaces()
    {
        if( !isset($this->resources['BW']))
        {
            $name = $this->getName();
            $this->resources['BW'] = $this->driver->getHostBandwidth($name);
        }
        $config = $this->getHostInterfaces();
        $metric = $this->resources['BW'];

        $tmp = array();
        foreach($metric as $row)
        {
            $tmp[$row['name']] = $row;
        }
        $metric = $tmp;

        $ret = array();
        $ret['global']['rx'] = 0;
        $ret['global']['tx'] = 0;
        foreach($config as $row)
        {
            $retRow = array();
            $retRow['device'] = $row['Name'];

            $addr_list = array();
            foreach($row['addresses'] as $addr)
            {
                //Ipv6 est représenté ainsi : fe80::7f75:ecf1:de89:f1c7%18. Il faut enlever la fin (18 est le ifindex de l'interface)
                $addr_list[] = explode('%',$addr['IPAddress'])[0];
            }
            $retRow['address'] = implode(',',$addr_list);
            $retRow['mac'] = $row['LinkLayerAddress'];
            $retRow['mtu'] = $row['MtuSize'];
            $retRow['speed'] = $row['Speed']/10000000;
            $retRow['rx'] = round($metric[$row['Name']]['rx'],0);
            $retRow['tx'] = round($metric[$row['Name']]['tx'],0);

            $ret[$retRow['device']] = $retRow;

            $ret['global']['rx'] += $retRow['rx'];
            $ret['global']['tx'] += $retRow['tx'];
        }
        return $ret;
    }

    public function getVMs():array
    {
        if( $this->vms_list == NULL )
        {
            $this->vms_list = array();
            $resources = $this->driver->getVMList($this->getName());
            foreach($resources['data'] as $vm_raw)
            {
                $retRow['request_time'] = $resources['request_time'];
                $retRow['data'] = $vm_raw;
                $this->vms_list[$vm_raw['vmid']] = new HyperVVM($retRow,$this->driver,$this);
            }
        }
        return array_values($this->vms_list);
    }
}

class HyperVStorage extends Storage
{
    protected $cluster;
    protected $hosts = NULL;
    protected $driver = NULL;
    protected $main_info;

    public function __construct($main_info,$driver,$cluster)
    {
        $this->cluster = $cluster;
        $this->hosts = $main_info['hosts'];
        $this->main_info = $main_info;
        $this->driver = $driver;
    }

    public function getId()
    {
        $simpleId = $this->main_info['data']['id'];
        if(preg_match('/\{([a-fA-F0-9\-]+)\}/', $simpleId, $matches))
        {
            $simpleId = $matches[1];
        }
        return $simpleId;
    }

    public function getName()
    {
        $main_info = $this->main_info['data'];
        if( empty($main_info['name']) )
        {
            if( !empty($main_info['letter']) )
            {
                return $main_info['path'];
            }
            else
            {
                $name = 'partition '.$main_info['PartitionNumber'] . ' disk '.$main_info['DiskNumber'];
                return $name;
            }
        }
        return $main_info['name'];
    }

    public function getHosts()
    {
        return $this->hosts;
    }

    public function getSizeTotal()
    {
        return $this->main_info['data']['size'];
    }

    public function getSizeUsed()
    {
        return $this->main_info['data']['size'] - $this->main_info['data']['free'];
    }

    public function getSizeVirtual()
    {
        return $this->main_info['data']['size'];
    }

    public function isShared()
    {
        return $this->main_info['data']['isSharedVolume'];
    }

    public function getType()
    {
        if( $this->main_info['data']['srtype'] == 'Unknown' )
        {
            return 'Unknown/Offline';
        }
        return $this->main_info['data']['srtype'];
    }

    public function getContent()
    {
        $main_info = $this->main_info['data'];

        if( $main_info['isQuorumDisk'] )
        {
            if( $this->main_info['data']['srtype'] == 'Unknown' )
            {
                return 'Cluster Quorum/Offline';
            }
            return 'Cluster Quorum';
        }
        elseif( $this->main_info['data']['srtype'] == 'Unknown' )
        {
            return 'Unknown/Offline';
        }
        elseif( $main_info['srtype'] == 'CDFS' )
        {
            return 'CD/DVD';
        }
        elseif( $main_info['type'] != 'Basic' )
        {
            return 'OS '.$main_info['type'];
        }
        elseif( $main_info['isSharedVolume'] )
        {
            return 'cluster shared VHD';
        }

        $pathList = $this->cluster->getStoragesPathList();
        $host = $this->getHosts();
        $host = array_shift($host);
        $name_host = $host->getName();
        $pathList = $pathList[$name_host]['data'];

        foreach($pathList as $row)
        {
            if( $row['VhdPath'] == $main_info['path'])
            {
                return 'hyperv VHD';
            }
        }

        if( $main_info['isBoot'])
        {
            return 'Windows system files';
        }
        elseif( in_array($main_info['srtype'],array('NTFS','FAT32')))
        {
            return 'local files';
        }
        else
        {
            return 'unknown';
        }
    }

    public function getDiskRead(){
        return 0;
    }

    public function getDiskWrite(){
        return 0;
    }

    public function getVolumes($vmid=NULL)
    {
        $main_info = $this->main_info['data'];
        $host = $this->getHosts();
        $host = array_shift($host);

        $pathList = $this->cluster->getStoragesPathList();
        $vhdList = $this->cluster->getVHDList();
        $volumes_list = array();


        $hosts_list = $this->getHosts();
        foreach($hosts_list as $host)
        {
            $host_name = $host->getName();
            $current_sr_path = $main_info['path'];

            $current_pathList = NULL;
            foreach($pathList[$host_name]['data'] as $row)
            {
                if( $current_sr_path == $row['PartitionPath'])
                {
                    $current_pathList = $row;
                    break;
                }
            }
            if( isset($current_pathList['VhdPath']) )
            {
                $current_vhd_path = $current_pathList['VhdPath'];
                foreach($vhdList[$host_name]['data'] as $row)
                {
                    if( trim($current_vhd_path,'\\') == trim($row['directoryPath'],'\\'))
                    {
                        $volumes_list[$row['Id']] = $row;
                    }
                }
            }
        }


        $ret = array();
        foreach($volumes_list as $row)
        {
            if( isset($vmid) && $row['VMId'] != $vmid )
            {
                continue;
            }
            $retRow = array();
            $retRow['id'] = $row['Path'];
            $retRow['name'] = $row['Name'];
            $retRow['desc'] = $row['Path'];
            $retRow['size_used'] = $row['FileSize'];
            $retRow['size_virt'] = $row['Size'];
            $retRow['parent'] = $row['ParentPath'];
            $retRow['storage'] = $this;
            $retRow['io_read']= 0;
            $retRow['io_write']= 0;
            $retRow['vm'] = $this->cluster->getVM($row['VMId']);
            $ret[] = $retRow;
        }
        return $ret;
    }
}

class HyperVVM extends VM
{
    protected $host;
    protected $driver = NULL;
    protected $main_info;

    private $resources = array();

    private static $enum_power_states = array(
        'Running' => 'running',
        'Off' => 'stopped',
        'Paused'  => 'paused',
    );

    public function __construct($main_info,$driver,$host)
    {
        $this->host = $host;
        $this->main_info = $main_info;
        $this->driver = $driver;

        $this->setId($this->main_info['data']['vmid']);
        $this->setName($this->main_info['data']['name']);
        $this->setType('VM');
        $this->setManufacturer('Microsoft');
    }

    private function getVMValues()
    {
        $vmid = $this->getId();
        return $this->host->getVMValues($vmid);
    }

    private function getVMSnapshots()
    {
        $vmid = $this->getId();
        return $this->host->getVMSnapshots($vmid);
    }

    private function getVMMetrics()
    {
        $vmid = $this->getId();
        return $this->host->getVMMetrics($vmid);
    }

    private function getVMAgent()
    {
        $resources = $this->getVMValues();
        return $resources['agent'] ?? FALSE;
    }

    private function getVMHASettings()
    {
        if( !$this->isHA())
        {
            return '';
        }
        else
        {
            $resources = $this->getVMValues();
            return $resources['ha_settings'] ?? '';
        }
    }

    public function getState()
    {
        $resources = $this->getVMValues();
        return self::$enum_power_states[$resources['state']];
    }

    public function getHost()
    {
        return $this->host;
    }

    public function isAlive()
    {
        $resources = $this->getVMValues();
        return $resources['alive'] ?? FALSE;
    }

    public function getHASettings(){
        $resources = $this->getVMAgent();
        return $resources['ha_settings'] ?? '';
    }

    public function isHA(){
        $resources = $this->getVMValues();
        return $resources['ha_enabled'] ?? FALSE;
    }

    public function getDescription()
    {
        $resources = $this->getVMValues();
        return $resources['description'] ?? '';
    }

    public function isLocked($refresh = false)
    {
        $resources = $this->getVMValues();
        return $resources['isLocked'];
    }

    public function getLockStatus($refresh = false)
    {
        $resources = $this->getVMValues();
        return $resources['lock_status'] ?? '';
    }

    public function getOS()
    {
        $resources = $this->getVMValues();
        return $resources['os'] ?? '';
    }

    public function getOSVersion()
    {
        $resources = $this->getVMValues();
        return $resources['os_version'] ?? '';
    }

    public function getUpTime()
    {
        $resources = $this->getVMValues();
        if( empty($resources['uptime']) )
        {
            return 0;
        }
        $time = time();
        $request_time = floor(array_sum($resources['request_time'])/2);
        $last_exec = $time - $request_time;
        return $resources['uptime'] + $last_exec;
    }

    public function getCpuPlatform()
    {
        $resources = $this->getVMValues();
        if(isset($resources['cpu_platform']))
        {
            return json_encode($resources['cpu_platform']);
        }
        return '';
    }

    public function isAgent()
    {
        $resources = $this->getVMAgent();
        return $resources['is_agent'] ?? FALSE;
    }

    public function getCpusTotal()
    {
        $resources = $this->getVMValues();
        return $resources['cpu_total'];
    }

    public function getCpusUsage()
    {
        $resources = $this->getVMMetrics();
        if( empty($resources))
        {
            return 0;
        }
        return round(($resources['AvgCPU']/100),2);
    }

    public function getDiskRead()
    {
        $resources = $this->getVMMetrics();
        if( empty($resources))
        {
            return 0;
        }
        return round($resources['AggregatedDiskDataRead'],2);
    }

    public function getDiskWrite()
    {
        $resources = $this->getVMMetrics();
        if( empty($resources))
        {
            return 0;
        }
        return round($resources['AggregatedDiskDataWritten'],2);
    }

    public function getNetInUsage(){
        $resources = $this->getVMMetrics();
        if( empty($resources))
        {
            return 0;
        }
        $sum = 0;
        foreach($resources['NetworkMeteredTrafficReport'] as $row)
        {
            if( $row['Direction'] == 1 )
            {
                $sum+=$row['TotalTraffic'];
            }
        }
        return round((($sum*8*1024)/ $resources['MeteringDuration']['TotalSeconds']),0);
    }

    public function getNetOutUsage(){
        $resources = $this->getVMMetrics();
        if( empty($resources))
        {
            return 0;
        }
        $sum = 0;
        foreach($resources['NetworkMeteredTrafficReport'] as $row)
        {
            if( $row['Direction'] == 2 )
            {
                $sum+=$row['TotalTraffic'];
            }
        }
        return round((($sum*8*1024)/ $resources['MeteringDuration']['TotalSeconds']),0);
    }

    public function getMemTotal()
    {
        $resources = $this->getVMValues();
        return $resources['mem_total'] ?? 0;
    }

    public function getMemUsage()
    {
        $resources = $this->getVMValues();
        return $resources['mem_usage'] ?? 0;
    }

    public function getSnapshots()
    {
        $resources = $this->getVMSnapshots();
        $ret = array();
        foreach($resources as $row)
        {
            $retRow = array();
            $retRow['id'] = $row['Id'];
            $retRow['name'] = $row['Name'];
            $retRow['description'] = $row['Notes'];
            $retRow['parent'] = $row['ParentSnapshotId'];

            preg_match('/\d+/', $row['CreationTime'], $matches);
            $retRow['time'] = round($matches[0]/1000,0);
            $ret[] = $retRow;
        }
        return $ret;
    }

    public function getVolumes()
    {
        $storages = $this->host->getCluster()->getStorages();
        $volumes_list = array();
        foreach($storages as $sr)
        {
            $sr_vol = $sr->getVolumes($this->getId());
            foreach($sr_vol as $vol)
            {
                $volumes_list[$vol['id']] = $vol;
            }
        }
        return array_values($volumes_list);
    }
}
