<?php
include_once '/usr/local/esia/plugins/lib/php/HTTP.class.php';
require_once '/usr/local/esia/plugins/lib/php/CacheFile.class.php';

class HypervClient
{
	//Partie HyperV
	private static $DEFAULT_CONF_FILE = '/etc/esia/rcm.ini';
	private static $DEFAULT_SRV_URL = 'http://127.0.0.1:2082';
	private static $PWSH_SCRIPT_PATH = '/usr/local/esia/public_html/plugin/virtualization/drivers';

	private static $winHostTCPTestTimeout = 3;
	private static $rcmConnectionTimeout = 3;
	private static $rcmRequestTimeout= 50;

	protected $httpClient= NULL;
	protected $postParams=array();

	private $isOpen = FALSE;

	//Partie CacheFile
	public static $TMP_DIR = '/tmp/esia/virt';

	public static $PERMANENT_TIME = 1800; //1h
	public static $LONG_TIME = 300; //5 min
	public static $SHORT_TIME = 60; //1 min

	//Pour les metric uniquement - On essaie d'atteindre cette valeur pour la periode
	public static $MIN_DIFF_TIME = 300; //1 min

	protected $driver;
	protected $class_tmp_path = NULL;
	protected $cluster_tmp_path = NULL;

	protected $caches_list = array();

	//Partie CacheRam
	protected $resource = array();
	protected $host_addr =  NULL;

	private $isReacheable = NULL;

	public function __construct($settings)
	{
		$o_host = $settings['url'];
        $o_user = $settings['login'];
        $o_pass = $settings['password'];

		$this->host_addr = $o_host;

		$conf = parse_ini_file(self::$DEFAULT_CONF_FILE,true);
		$o_key = $conf['LISTEN']['key'];

		$params['key'] = $o_key;
		$params['action'] = 'run';
		$params['proto'] = 'ps1';
		$params['hostaddr'] = $o_host;
		$params['user'] = $o_user;
		$params['authentication'] = $o_pass;
		$this->postParams = $params;
		$this->httpClient = new ESIA\HTTP\HttpClient(self::$DEFAULT_SRV_URL);

		$this->class_tmp_path = $o_host;
	}

	protected function initConnection()
	{
		if( !$this->isOpen )
		{
			$this->httpClient->open();
			$this->httpClient->setOptions(CURLOPT_CONNECTTIMEOUT,self::$rcmConnectionTimeout);
			$this->httpClient->setOptions(CURLOPT_TIMEOUT,self::$rcmRequestTimeout);
			$this->isOpen = TRUE;
		}
	}

	public function isOpen()
	{
		return $this->isOpen;
	}

	public static function testPowershellPortAccess($host)
	{
		$ports = array(5985,5986);
		foreach ($ports as $port)
		{
			$connection = @fsockopen($host, $port,$errno, $errstr, self::$winHostTCPTestTimeout);
			if (is_resource($connection))
			{
				fclose($connection);
				return TRUE;
			}
		}
		return FALSE;
	}

	public function testPortAccess()
	{
		if( $this->isReacheable == NULL )
		{
			$this->isReacheable = self::testPowershellPortAccess($this->host_addr);
		}
		return $this->isReacheable;
	}

	public function execute($command,$withJson)
	{
		$isReacheable = $this->testPortAccess();
		if( !$isReacheable )
		{
			return array();
		}

		$this->initConnection(); //Si le driver est déjà ouvert, alors cette commande ne fera rien.
		$params = $this->postParams;
		if( !$withJson )
		{
			$params['command'] = $command;
			$data = $this->httpClient->post('',$params);
            $data = json_decode($data,TRUE);
			if( !empty($data['stderr']) )
			{
				throw new Exception($data['stderr']);
			}

			$data = $data['stdout'];
		}
		else
		{
			$params['command'] = "$command|ConvertTo-Json";
			$data = $this->httpClient->post('',$params);

            $data = json_decode($data,TRUE);
			if( empty($data['stdout']) && !empty($data['stderr']) )
			{
				throw new Exception("Powershell error :\n[\n".$data['stderr'] .$data['stdout']." : $command\n]");
			}
			$data = json_decode($data['stdout'],TRUE);
		}
		$code = $this->httpClient->getHttpCode();
		if(empty($data))
		{
			$data = array();
		}
		return $data;
	}

	public function withCacheExecute($command,$key,$during)
	{
		$during = $during/60;
		$cache = $this->getCache($key,$during*10);
		if( !$cache )
		{
			$resources['data'] = array();
			$time[] = time();
			$time[] = time();
			$resources['request_time'] = $time;
			$resources['source'] = 'none';
			return $resources;
		}
		if( $cache->isValid($during) && $during != 0)
		{
			$data = $cache->getData();
			$data = json_decode($data,TRUE);
			$data['source'] = 'cache';
			return $data;
		}
		else
		{
			$time = array();
			$time[] = time();
			$data = $this->execute($command,TRUE);
			$time[] = time();
			$resources['data'] = $data;
			$resources['request_time'] = $time;
			$cache->writeData(json_encode($resources));
			$resources['source'] = 'request';
			exec('chown -R www-data:www-data '.self::$TMP_DIR);
			return $resources;
		}
	}

	public function executeFile($header,$filename,$withJson)
	{
		$command = "$header\n";
		$command .= trim(file_get_contents(self::$PWSH_SCRIPT_PATH ."/$filename"));
		return $this->execute($command,$withJson);
	}

	public function withCacheExecuteFile($header,$filename,$key,$during)
	{
		$command = "$header\n";
		$command .= trim(file_get_contents(self::$PWSH_SCRIPT_PATH ."/$filename"));
		return $this->withCacheExecute($command,$key,$during);
	}

	private static function convertToArray($data)
	{
		if( !empty($data) && !is_numeric(array_key_first($data)) )
		{
			return array($data);
		}
		return $data;
	}

	protected function getCache($key,$during)
	{
		if( isset($this->caches_list[$key]))
		{
			return $this->caches_list[$key];
		}
		if( $key == 'cluster.info' )
		{
			$dir_path = self::$TMP_DIR.'/'.$this->class_tmp_path;
		}
		else
		{
			if( empty($this->cluster_tmp_path) )
			{
				//$resources = $this->getResourceData('cluster.info',self::$PERMANENT_TIME);
				$resources = $this->getCluster();
				$resources = $resources['data'];
				if( !isset($resources['Id']) )
				{
					return FALSE;
				}
				$id = $resources['Id'];
				$name = $resources['Name'];

				$id = explode('-',$id)[0];
				$name = preg_replace("/[^A-Za-z0-9 ]/", '', $name);
				$this->cluster_tmp_path = $name . '_'.$id;
			}
			$dir_path = self::$TMP_DIR.'/'.$this->cluster_tmp_path;
		}
		if (!file_exists($dir_path))
		{
			mkdir($dir_path, 0777, TRUE);
		}
		$file_path = "$dir_path/$key.dat";
		$this->caches_list[$key] = new ESIA\CacheFile\CacheFile($file_path,$during);
		return $this->caches_list[$key];
	}

	public function getCluster()
	{
		$key = 'cluster.info';
		$script = 'CLUSTER/Cluster_brief.ps1';
		return $this->withCacheExecuteFile('',"hyperv/pwsh/$script",$key,self::$PERMANENT_TIME);
	}

	private function saveHostListAddress($data)
	{
		$cache = $this->getCache('hostsAddrList',self::$PERMANENT_TIME);
		$dir_path = self::$TMP_DIR.'/'.$this->class_tmp_path;
		if (!file_exists($dir_path))
		{
			mkdir($dir_path, 0777, TRUE);
		}
		$file_path = "$dir_path/hostsAddrList.dat";

		$addrList = array();
		foreach($data as $row)
		{
			$addrList[] = $row['mainAddress'];
		}
		file_put_contents($file_path,json_encode($addrList));
	}

	public function getHostListAddress()
	{
		$cache = $this->getCache('hostsAddrList',self::$PERMANENT_TIME);
		$dir_path = self::$TMP_DIR.'/'.$this->class_tmp_path;
		if (!file_exists($dir_path))
		{
			mkdir($dir_path, 0777, TRUE);
		}
		$file_path = "$dir_path/hostsAddrList.dat";
		$data = FALSE;
		if( is_readable($file_path))
		{
			$data = file_get_contents($file_path);
			$data = json_decode($data,TRUE);
		}
		if( !$data )
		{
			return array();
		}
		return $data;
	}

	public function getHostList()
	{
		$key = 'hosts.list';
		$script = 'HOST/Host_brief.ps1';
		$data = $this->withCacheExecuteFile('',"hyperv/pwsh/$script",$key,HypervClient::$LONG_TIME);
		$data['data'] = self::convertToArray($data['data']);
		$this->saveHostListAddress($data['data']);
		return $data;
	}

	public function getHostValues($name)
	{
		$key = "hosts.values.$name";
		$script = 'HOST/Host.ps1';
		return $this->withCacheExecuteFile("\$ComputerName='$name'\n","hyperv/pwsh/$script",$key,HypervClient::$SHORT_TIME);
	}

	public function getHostInterfaces($name)
	{
		$key = "hosts.iface.$name";
		$script = 'HOST/Host_ifaces.ps1';
		return $this->withCacheExecuteFile('',"hyperv/pwsh/$script",$key,HypervClient::$LONG_TIME);
	}

	public function getCounterValue($filepath,$command,$max_age,$advised_interval,$validate)
	{
		//chargement des données sauvées sur le disque
		$from_disk_data = array();
		if( is_file($filepath) )
		{
			$fp = fopen($filepath, "r");
			$from_disk_data = fread($fp, filesize($filepath));
			flock($fp, LOCK_UN);
			$from_disk_data = json_decode($from_disk_data,TRUE);
			if( $from_disk_data == FALSE )
			{
				$from_disk_data = array();
			}
		}

		$current_time = time();
		$min_time = $current_time - $max_age;

		$to_save_data = array();
		foreach($from_disk_data as $ts => $data)
		{
			if( $ts > $min_time )
			{
				$to_save_data[$ts] = $data;
			}
		}

		//Si les 2 valeurs sont trop proches, on va quand même en récupérer 1.
		$nb_data = count($to_save_data);
		if( ($current_time - array_key_first($to_save_data)) <  $advised_interval)
		{
			$nb_data--;
		}

		if( $nb_data < 2 )
		{
			$sleep_time = 0;
			do
			{
				if( !$this->testPortAccess())
				{
					return FALSE;
				}

				$from_host_data = $this->execute($command,TRUE);
				validateArray($from_host_data,$validate);
				$to_save_data[time()] = $from_host_data;
				$sleep_time = 3;
				$nb_data++;
			}while($nb_data < 2);
		}

		//sauvegarde des données sur le disques
		if( !@file_put_contents($filepath,json_encode($to_save_data), LOCK_EX))
		{
			throw new Exception("Unable to save data into tmp file $filepath");
		}

		$first_key = array_key_first($to_save_data);
		$last_key = array_key_last($to_save_data);
		$ret['old'] = $to_save_data[$first_key];
		$ret['new'] = $to_save_data[$last_key];
		return $ret;
	}

	public function getHostCPU($name)
	{
		$max_age = self::$LONG_TIME;
		$advised_interval = self::$MIN_DIFF_TIME;

		//Formula : (1- ((Nx - N0) / (Dx - D0))) x 100
		//see : https://learn.microsoft.com/en-us/previous-versions/windows/it-pro/windows-server-2003/cc757283(v=ws.10)

		$type = 'CPU';
		$filepath = self::$TMP_DIR.'/'.$this->cluster_tmp_path .'/'."metric_$type.$name.dat";

		$select = 'PercentProcessorTime,Timestamp_Sys100NS';
		$query = 'select '.$select.' from Win32_PerfRawData_PerfOS_Processor where Name="_Total"';
		$command = "Get-CimInstance -Query '$query' | select -Property PercentProcessorTime,Timestamp_Sys100NS";

		$data = $this->getCounterValue($filepath,$command,$max_age,$advised_interval,$select);
		if( !$data )
		{
			return 0;
		}

		$cpu_timepercent = $data['new']['PercentProcessorTime'] - $data['old']['PercentProcessorTime'];
		$cpu_timestamp = $data['new']['Timestamp_Sys100NS'] - $data['old']['Timestamp_Sys100NS'];
		$cpu_usage = (1 - ($cpu_timepercent / $cpu_timestamp));
		$cpu_usage = round($cpu_usage,2);
		return $cpu_usage;
	}

	public function getHostBandwidth($name)
	{
		$max_age = self::$LONG_TIME;
		$advised_interval = self::$MIN_DIFF_TIME;

		$type = 'IFACE';
		$filepath = self::$TMP_DIR.'/'.$this->cluster_tmp_path .'/'."metric_$type.$name.dat";

		$stats = 'Bytes';
        $stats = explode(',',$stats);

        $stats_select = '';
        foreach($stats as $row)
        {
            $stats_select.= "Received$row,Sent$row,";
        }
        $stats_select = trim("Name,InstanceID,$stats_select",',');
		$command = "\$hash = @{};";
		$command.= "\$hash.start=(Get-Date).TimeOfDay.TotalSeconds;";
		$command.= "\$hash.data=Get-NetAdapterStatistics| select -Property $stats_select;";
		$command.= "\$hash.end=(Get-Date).TimeOfDay.TotalSeconds;";
        $command.= "\$hash";

		$data = $this->getCounterValue($filepath,$command,$max_age,$advised_interval,array('start','end','data'));
		if( !$data )
		{
			return array();
		}

		//Les infos sont par request, on transforme pour avoir les infos par interface
		$ret = array();
		foreach($data as $key => $arr)
		{
			$time = $arr['start'] + ($arr['end'] - $arr['start']) /2;
			foreach($arr['data'] as $row)
			{
				$ret[$row['InstanceID']]['name'] = $row['Name'];
				$ret[$row['InstanceID']]['time'][$key]  = $time;
				$ret[$row['InstanceID']]['rx'][$key]  = $row['ReceivedBytes'];
				$ret[$row['InstanceID']]['tx'][$key]  = $row['SentBytes'];
			}
		}
		$data = $ret;

		//On réalise les calculs
		$ret = array();
		foreach($data as $key => $row)
		{
			$retRow = array();
			$period = $row['time']['new'] - $row['time']['old'];

			$retRow['id'] =  $key;
			$retRow['name'] =  $row['name'];
			//On calcule la vistesse et on profite pour transformer en bits (bps)
			$retRow['rx'] = (($row['rx']['new'] - $row['rx']['old'])*8)/$period;
			$retRow['tx'] = (($row['tx']['new'] - $row['tx']['old'])*8)/$period;
			$ret[] = $retRow;
		}
		return $ret;
	}
/*
	public function getHostIODisks($name)
	{
		$max_age = self::$LONG_TIME;
		$advised_interval = self::$MIN_DIFF_TIME;

		//see : https://www.computerperformance.co.uk/powershell/perfmon/
    	//see : https://stackoverflow.com/questions/27272800/retrieve-average-disk-sec-per-transfer-using-wmi
    	//see : https://forums.powershell.org/t/win32-perfrawdata-perfdisk-logicaldisk/9876/4

		$type = 'PDISK';
		$filepath = self::$TMP_DIR.'/'.$this->cluster_tmp_path .'/'."metric_$type.$name.dat";

		$bytesPersec_keys = 'DiskReadBytesPersec,DiskWriteBytesPersec';
		$select = "Name,Timestamp_PerfTime,Frequency_PerfTime,$bytesPersec_keys";
	    $query = 'Select '.$select.' from Win32_PerfRawData_PerfDisk_PhysicalDisk';
		$command = "Get-CimInstance -Query '$query' | select -Property $select";

		$data = $this->getCounterValue($filepath,$command,$max_age,$advised_interval,array(0));

		//((Nx - N0) / ((Dx - D0)/ F)
		$ret = array();
		foreach($data['old'] as $row)
	 	{
			$ret[$row['Name']]['old'] = $row;
		}
		foreach($data['new'] as $row)
		{
			$ret[$row['Name']]['new'] = $row;
		}
		$data = $ret;

		$ret = array();
		foreach($data as $key => $row)
		{
			$old_row = $row['old'];
			$new_row = $row['new'];

			$retRow = array();
			$retRow['Name'] = $key;

			foreach(explode(',',$bytesPersec_keys) as $p_key)
			{

				$n0 = $old_row[$p_key];
				$n1 = $new_row[$p_key];
				$d0 = $old_row['Timestamp_PerfTime'];
				$d1 = $new_row['Timestamp_PerfTime'];
				$f = $new_row['Frequency_PerfTime'];

				//((Nx - N0) / ((Dx - D0)/ F)
				$retRow[$p_key] = ($n1-$n0)/(($d1-$d0)/$f);
				$retRow[$p_key] = round($retRow[$p_key],2);
			}
			$ret[] = $retRow;
		}
		return $ret;
	}
*/
	public function getVMList($name)
	{
		$key = "vms.list.$name";
		$script = 'VM/VM_brief.ps1';
		$data = $this->withCacheExecuteFile("\$ComputerName='$name'\n","hyperv/pwsh/$script",$key,HypervClient::$LONG_TIME);
		$data['data'] = self::convertToArray($data['data']);

		return $data;
	}

	public function getSRList($name)
	{
		$key = "storages.list.$name";
		$script = 'SR/SR_brief.ps1';
		$data = $this->withCacheExecuteFile('',"hyperv/pwsh/$script",$key,HypervClient::$LONG_TIME);
		$data['data'] = self::convertToArray($data['data']);
		return $data;
	}

	public function getSRVHD($name)
	{
		$key = "storages.vhd.$name";
		$script = 'SR/VHD.ps1';
		$data = $this->withCacheExecuteFile("\$ComputerName='$name'\n","hyperv/pwsh/$script",$key,HypervClient::$SHORT_TIME);
		$data['data'] = self::convertToArray($data['data']);
		return $data;
	}

	public function getSRPath($name)
	{
		$key = "storages.path.$name";
		$script = 'SR/SR_path.ps1';
		$data = $this->withCacheExecuteFile("\$ComputerName='$name'\n","hyperv/pwsh/$script",$key,HypervClient::$LONG_TIME);
		$data['data'] = self::convertToArray($data['data']);
		return $data;
	}

	public function getVMValues($name)
	{
		$key = "vms.values.$name";
		$script = 'VM/VM.ps1';
		//Retourne les infos de toutes les VMs de l'hyperviseur
		$data = $this->withCacheExecuteFile("\$ComputerName='$name'\n","hyperv/pwsh/$script",$key,HypervClient::$SHORT_TIME);
		return $data;
	}

	public function getVMSnapshots($name)
	{
		$key = "vms.snapshots.$name";
		$script = 'VM/Snapshots.ps1';
		//Retourne les infos de tous les snapshots des VMs de l'hyperviseur

		$data = $this->withCacheExecuteFile("\$ComputerName='$name'\n","hyperv/pwsh/$script",$key,HypervClient::$SHORT_TIME);
		$data['data'] = self::convertToArray($data['data']);
		return $data;
	}

	public function getVMMetrics($name)
	{
		$key = "vms.metrics.$name";
		$script = 'VM/VM_metrics.ps1';
		$data =  $this->withCacheExecuteFile("","hyperv/pwsh/$script",$key, HypervClient::$LONG_TIME);
		if( $data['source'] == 'cache' )
		{
			foreach($data['data'] as $row)
			{
				if( !empty($row['MeteringDuration']['TotalSeconds'])  && $row['MeteringDuration']['TotalSeconds'] <  HypervClient::$MIN_DIFF_TIME)
				{
					return $this->withCacheExecuteFile("","hyperv/pwsh/$script",$key,0);
				}
			}
		}
		return $data;
	}
}

function validateArray($arr,$keys)
{
    if( !is_array($arr))
    {
        throw new Exception("Unable to parse received data from host.");
    }
    if( !is_array($keys) )
    {
        $keys = explode(',',$keys);
    }
    foreach($keys as $key)
    {
        if( !array_key_exists($key,$arr))
        {
			print_r($arr);
            throw new Exception("Missing key '$key' from received data from host.");
        }
    }
}
